@@ -307,8 +307,7 @@ function generate_random_materials()
         local cind=1
         local pos_sphere={metal={},silk={}}
         local have={metal={},silk={}}
-
-        for k,v in pairs(world.spheres) do
+        for k,v in pairsByKeys(world.spheres) do
             pos_sphere.metal[#pos_sphere.metal+1]=k
             pos_sphere.silk[#pos_sphere.silk+1]=k
         end
@@ -342,7 +341,7 @@ function generate_random_materials()
             end
             we_have_made_enough_divine_mats=true
             --IS ALL OF THE COSMOS REPRESENTED OR AT LEAST NEUTRAL TO THE SITUATION?
-            for k,v in pairs(world.spheres) do
+            for k,v in pairsByKeys(world.spheres) do
                 local met,slk=false,false
                 for kk,vv in pairs(have.metal) do
                     met=met or not v.enemies[kk]
@@ -380,7 +379,7 @@ function generate_random_materials()
     end
     if random_object_parameters.use_mythical_materials then
         local cind=1
-        for sph,_ in pairs(random_object_parameters.mythical_sphere) do
+        for sph,_ in pairsByKeys(random_object_parameters.mythical_sphere) do
             local res=generate_from_list(materials.mythical_remnant,sph)
             local generated_tbl = res and (res.mat or res.raws)
             if generated_tbl then
@@ -418,11 +417,13 @@ end
 
 function generate_random_languages()
     local lines={}
-    for k,v in pairs(languages) do
+    for k,v in pairsByKeys(languages) do
         lines[#lines+1]="[TRANSLATION:"..k.."]"
         add_generated_info(lines)
-        for kk,vv in pairs(v()) do
-            lines[#lines+1]="[T_WORD:"..kk..":"..vv.."]"
+        local language_by_word=v()
+        for kk,vv in ipairs(world.language.word) do
+            local tok=vv.token
+            lines[#lines+1]="[T_WORD:"..tok..":"..language_by_word[tok].."]"
         end
     end
     raws.register_languages(lines)
@@ -564,7 +565,6 @@ function generate_random_creatures()
             failed_large="E_FL",
         }
         local tok=ropar.token_prefix..exp_token[ropar.making_experiment]..tostring(ropar.experimenter_experiment_count+1)
-        local orig_debug_level=debug_level
         local c=generate_from_list(creatures.experiment[ropar.making_experiment],tok)
         local generated_tbl = c and (c.creature or c.raws)
         if generated_tbl then
@@ -771,8 +771,8 @@ function generate_random_interactions()
     if ropar.use_mythical_materials then
         local pind=1
         local allowed_interactions={}
-        for sph,_ in pairs(ropar.mythical_sphere) do
-            for k,v in pairs(interactions.mythical_item_power) do
+        for sph,_ in pairsByKeys(ropar.mythical_sphere) do
+            for k,v in pairsByKeys(interactions.mythical_item_power) do
                 if v.spheres[sph] then
                     allowed_interactions[#allowed_interactions+1]=v
                 end
