@@ -303,7 +303,7 @@ creatures.demon.default=function(demon_type,difficulty,tok)
 
 [CAN_LEARN][CAN_SPEAK][DIFFICULTY:10]
         ]])
-        options.humanoid_only=true
+        options.humanoidable_only=true
         options.can_learn=true
     else
         tbl=split_to_lines(tbl,[[
@@ -318,7 +318,7 @@ creatures.demon.default=function(demon_type,difficulty,tok)
 [NATURAL_SKILL:DODGING:10]
 [NATURAL_SKILL:SITUATIONAL_AWARENESS:10]
         ]])
-        if(demon_type=="humanoid_beast") then options.humanoid_only=true end
+        if(demon_type=="humanoid_beast") then options.humanoidable_only=true end
         if one_in(2) then 
             tbl[#tbl+1]="[CAN_LEARN][CAN_SPEAK]"
             options.can_learn=true
@@ -439,7 +439,7 @@ creatures.demon.default=function(demon_type,difficulty,tok)
             end
         end
         if #candidates==0 then
-            for k,v in pairs(available_sphere_cur) do
+            for k,v in pairsByKeys(available_sphere_cur) do
                 if v then candidates[#candidates+1]=k end
             end
         end
@@ -448,7 +448,7 @@ creatures.demon.default=function(demon_type,difficulty,tok)
             add_sphere_mpp(options.spheres,new_s,available_sphere,available_sphere_cur)
             while one_in(2) do
                 candidates={}
-                for k,v in pairs(world.spheres[new_s].friends) do
+                for k,v in pairsByKeys(world.spheres[new_s].friends) do
                     if available_sphere_cur[k] then
                         candidates[#candidates+1]=k
                     end
@@ -515,7 +515,6 @@ end
 
 function night_creature_universals(lines,options)
     map_merge(options,{
-    humanoid_only=true,
     do_not_make_uniform=true,
     always_nobreathe=true,
     no_random_attack_tweak=true,
@@ -978,7 +977,6 @@ creatures.night_creature.nightmare.default=function(tok)
         token=tok
     }
     night_creature_universals(lines,options)
-    options.humanoid_only=false
     options.cannot_have_antennae=false
     if one_in(10) then options.glowing_eyes=true end
     lines[#lines+1]="[NIGHT_CREATURE_NIGHTMARE]"
@@ -1139,6 +1137,7 @@ creatures.night_creature.werebeast.default=function(tok)
         animal_coloring_allowed=true,
         no_tweak=true,
         material_weakness=true,
+        humanoidable_only=true,
         beast_only=true,
         prioritize_bite=true,
         token=tok
@@ -1252,12 +1251,16 @@ function experiment_tokens(lines,options)
     lines[#lines+1]="[NIGHT_CREATURE]"
     lines[#lines+1]="[SUPERNATURAL]"
     lines[#lines+1]="[FANCIFUL]"
-    if not one_in(4) and random_object_parameters.experimenter_creature.flags.MATES_TO_BREED then
-        lines[#lines+1]="[CASTE:FEMALE]"
-            lines[#lines+1]="[FEMALE]"
-        lines[#lines+1]="[CASTE:MALE]"
-            lines[#lines+1]="[MALE]"
-        lines[#lines+1]="[SELECT_CASTE:ALL]"
+    if not one_in(4) and random_object_parameters.experimenter_creature then
+        if random_object_parameters.experimenter_creature.flags.MATES_TO_BREED then
+            lines[#lines+1]="[CASTE:FEMALE]"
+                lines[#lines+1]="[FEMALE]"
+            lines[#lines+1]="[CASTE:MALE]"
+                lines[#lines+1]="[MALE]"
+            lines[#lines+1]="[SELECT_CASTE:ALL]"
+        else
+            lines[#lines+1]="[NO_GENDER]"
+        end
     else
         lines[#lines+1]="[NO_GENDER]"
     end
@@ -1291,11 +1294,13 @@ end
 
 function experimenter_sphere_info(lines,options)
     local ropar=random_object_parameters
-    if ropar.experimenter_creature.caste[ropar.experimenter_hf.caste].flags.UNIQUE_DEMON then
-        -- ?. would be pretty cool, alas
-        local spheres = ropar.experimenter_hf.profile and ropar.experimenter_hf.profile.mpp and ropar.experimenter_hf.profile.mpp.spheres
-        if spheres then
-            return spheres.CHAOS,spheres.DEFORMITY
+    if ropar.experimenter_creature then
+        if ropar.experimenter_creature.caste[ropar.experimenter_hf.caste].flags.UNIQUE_DEMON then
+            -- ?. would be pretty cool, alas
+            local spheres = ropar.experimenter_hf.profile and ropar.experimenter_hf.profile.mpp and ropar.experimenter_hf.profile.mpp.spheres
+            if spheres then
+                return spheres.CHAOS,spheres.DEFORMITY
+            end
         end
     end
     return false,false
@@ -1350,6 +1355,8 @@ function experiment_description(lines,options)
                     else
                         add_desc(ropar.experiment_source_race_name)
                     end
+                    add_desc(" ")
+                    add_desc(ropar.experiment_hf.name.translated)
                 else add_desc("an unknown creature")
                 end
             elseif ropar.experimenter_source_race~=-1 then
@@ -1422,7 +1429,7 @@ function experiment_name_token(lines,options)
     local experiment_name=nil
     if name_to_use then
         -- we preprocess these indices for lua--i know it's inconsistent, sorry
-        if string.len(name_to_use.firstname)>0 and ((name_to_use.words.FRONT_COMPOUND.index==0 and name_to_use.THE.index==0) or one_in(2)) then
+        if string.len(name_to_use.firstname)>0 and ((not name_to_use.translated_epithet_the and not name_to_use.translated_epithet_compound) or one_in(2)) then
             experiment_name=name_to_use.firstname
             experiment_name=capitalize_string_words(experiment_name)
         else
@@ -1716,7 +1723,7 @@ failed_experiment_rcps={
     {
         name_string="armless biped",
         tile='b',
-        body_base="TWO_LEGS_NO_ARMS",
+        body_base="TWO_LEGS_NO_ARMS_HUMANOID",
         always_flightless=true
     },
 }
@@ -2025,7 +2032,7 @@ function angel_end_tokens(lines,options,names_tbl)
     for k,v in pairs(options.spheres) do
         if v then cand[#cand+1]=k end
     end
-    if #cand>0 and (not add_name or one_in(2)) then
+    if #cand>0 and (string.len(add_name)==0 or one_in(2)) then
         proper_name=false
         if one_in(2) then
             have_adjective=true
@@ -2068,7 +2075,6 @@ end
 creatures.angel.humanoid_warrior.default=function(tok)
     local lines={}
     local options={
-        humanoid_only=true,
         can_learn=true,
         do_sphere_rcm=true,
         pick_sphere_rcm=true,
@@ -2103,6 +2109,7 @@ creatures.angel.humanoid_warrior.default=function(tok)
     populate_sphere_info(lines,options)
     local rcm
     if one_in(4) then
+        options.humanoidable_only=true
         rcp=get_random_creature_profile(options)
     else
         rcp={
@@ -2143,7 +2150,6 @@ end
 creatures.angel.humanoid_generic.default=function(tok)
     local lines={}
     local options={
-        humanoid_only=true,
         do_sphere_rcm=true,
         pick_sphere_rcm=true,
         force_ichor=true,
@@ -2176,6 +2182,7 @@ creatures.angel.humanoid_generic.default=function(tok)
     populate_sphere_info(lines,options)
     local rcm
     if one_in(4) then
+        options.humanoidable_only=true
         rcp=get_random_creature_profile(options)
     else
         rcp={
