@@ -170,13 +170,13 @@ not_beast_random_creature={
     GENERAL_QUADRUPED=true,
 }
 
-function is_valid_random_creature(creature, do_water, humanoid_only, is_good, beast_only)
+function is_valid_random_creature(creature, do_water, humanoidable_only, is_good, beast_only)
 	local l3=get_debug_logger(3)
     if not do_water and water_based_random_creature[creature] then 
 		l3("Water failure")
 		return false
 	end
-    if humanoid_only and not humanoidable_random_creature[creature] then 
+    if humanoidable_only and not humanoidable_random_creature[creature] then 
 		l3("Humanoid failure")
 		return false
 	 end
@@ -194,39 +194,45 @@ end
 
 random_creature_types_finalized=false -- global on purpose, users should be able to set this to false
 
+random_creature_types_presorted={} -- Also global on purpose, in case modders find some horrible way to use this too
+
 function finalize_random_creature_types()
-	if random_creature_types_finalized then return end
-	random_creature_types_finalized=true
-	for k,v in pairs(random_creature_types) do
-		v.type=k
-		if v.front_digits then
+    if random_creature_types_finalized then return end
+    random_creature_types_finalized=true
+    for k,v in pairs(random_creature_types) do
+        v.type=k
+        if v.front_digits then
             v.front_digits=math.min(v.front_digits,5)
             v.front_digits=math.max(v.front_digits,2)
             v.front_digits=math.floor(v.front_digits)
-		end
-		if v.rear_digits then
+        end
+        if v.rear_digits then
             v.rear_digits=math.min(v.rear_digits,5)
             v.rear_digits=math.max(v.rear_digits,2)
             v.rear_digits=math.floor(v.rear_digits)
-		end
-		if humanoidable_random_creature[k] then v.humanoidable=true end
-	end
+        end
+        if humanoidable_random_creature[k] then v.humanoidable=true end
+        random_creature_types_presorted[#random_creature_types_presorted+1]={k,v}
+    end
+    table.sort(random_creature_types_presorted,function(a,b) return a[1]<b[1] end)
 end
 
 function get_random_creature_profile(options,blacklist)
-	local l3=get_debug_logger(3)
-	local l5=get_debug_logger(4)
-	finalize_random_creature_types()
+    local l3=get_debug_logger(3)
+    local l5=get_debug_logger(4)
+    finalize_random_creature_types()
     local candidates={}
-    for k,v in pairs(random_creature_types) do
-        if (not blacklist or not blacklist[k]) and is_valid_random_creature(k,options.do_water,options.humanoid_only,options.is_good,options.beast_only) then
+    for i,tbl in ipairs(random_creature_types_presorted) do
+        local k=tbl[1]
+        local v=tbl[2]
+        if (not blacklist or not blacklist[k]) and is_valid_random_creature(k,options.do_water,options.humanoidable_only,options.is_good,options.beast_only) then
             candidates[#candidates+1]=v
-			l5("Succeeded!",k)
-		else
-			l3("Failed! ",k,(not blacklist or not blacklist[k]))
-		end
+            l5("Succeeded!",k)
+        else
+            l3("Failed! ",k,(not blacklist or not blacklist[k]))
+        end
     end
-	return pick_weighted_from_table(candidates) or random_creature_types.GENERAL_BLOB
+    return pick_weighted_from_table(candidates) or random_creature_types.GENERAL_BLOB
 end
 
 local function quadruped_function_curry(is_grasp) -- thank you lua (or maybe don't?)
@@ -242,12 +248,12 @@ local function quadruped_function_curry(is_grasp) -- thank you lua (or maybe don
 	end
 	return function(rcp,options)
 		local bp={"RCP_UPPER_BODY","RCP_LOWER_BODY","RCP_NECK","RCP_HEAD"}
-		if not options.tweak and not rcp.cannot_have_get_more_legs and one_in(20) then
+		if not options.btc and not rcp.cannot_have_get_more_legs and one_in(20) then
 			if one_in(2) then
-				options.tweak="EIGHT_LEGGED"
+				options.btc="EIGHT_LEGGED"
 				table_merge(bp,{first_leg,"RCP_SECOND_SIMPLE_LEGS","RCP_THIRD_SIMPLE_LEGS","RCP_FOURTH_SIMPLE_LEGS"})
 			else
-				options.tweak="SIX_LEGGED"
+				options.btc="SIX_LEGGED"
 				table_merge(bp,{first_leg,"RCP_SECOND_SIMPLE_LEGS","RCP_THIRD_SIMPLE_LEGS"})
 			end
 		else
@@ -255,10 +261,10 @@ local function quadruped_function_curry(is_grasp) -- thank you lua (or maybe don
 		end
 		if rcp.must_have_mantis_arms then
 			options.pcg_layering_base="BEAST_INSECT"
-		elseif rcp.cannot_have_get_more_legs and not rcp.slinky_quadruped then
-			options.pcg_layering_base=pick_random({"BEAST_QUADRUPED_SLINKY","BEAST_QUADRUPED_BULKY"})
 		elseif is_grasp then
 			options.pcg_layering_base="BEAST_FRONT_GRASP"
+		elseif rcp.cannot_have_get_more_legs and not rcp.slinky_quadruped then
+			options.pcg_layering_base=pick_random({"BEAST_QUADRUPED_SLINKY","BEAST_QUADRUPED_BULKY"})
 		elseif rcp.slinky_quadruped then
 			options.pcg_layering_base="BEAST_QUADRUPED_SLINKY"
 		else
@@ -315,6 +321,19 @@ body_base_fun=body_base_fun or {
 		options.walk_speed=900
 		return {"RCP_BASIC_BODY","RCP_NECK","RCP_HEAD","RCP_FIRST_SIMPLE_LEGS","RCP_SECOND_SIMPLE_LEGS","RCP_THIRD_SIMPLE_LEGS","RCP_FOURTH_SIMPLE_LEGS"}
 	end,
+	TEN_LEGGED_LOBSTERFORM=function(rcp,options)
+		options.pcg_layering_base="BEAST_SCORPION"
+		options.walk_var="STANDARD_WALKING_GAITS"
+		options.walk_speed=900
+		return {"RCP_BASIC_BODY","RCP_NECK","RCP_HEAD",
+		"RCP_FIRST_SIMPLE_LEGS","RCP_SECOND_SIMPLE_LEGS","RCP_THIRD_SIMPLE_LEGS","RCP_FOURTH_SIMPLE_LEGS","RCP_FIFTH_SIMPLE_LEGS"}
+	end,
+	EIGHT_LEGGED_LOBSTERFORM=function(rcp,options)
+		options.pcg_layering_base="BEAST_SCORPION"
+		options.walk_var="STANDARD_WALKING_GAITS"
+		options.walk_speed=900
+		return {"RCP_BASIC_BODY","RCP_NECK","RCP_HEAD","RCP_FIRST_SIMPLE_LEGS","RCP_SECOND_SIMPLE_LEGS","RCP_THIRD_SIMPLE_LEGS","RCP_FOURTH_SIMPLE_LEGS"}
+	end,
 	WORM=function(rcp,options)
 		options.pcg_layering_base="BEAST_WORM_LONG"
 		options.walk_var="STANDARD_WALK_CRAWL_GAITS"
@@ -322,7 +341,7 @@ body_base_fun=body_base_fun or {
 		return {"RCP_BASIC_BODY_STANCE_WITH_HEAD_FLAG"}
 	end,
 	NO_LIMB=function(rcp,options)
-		options.pcg_layering_base="BEAST_BIPEDAL_DINOSAUR"
+		options.pcg_layering_base="BEAST_WALRUS"
 		options.walk_var="STANDARD_WALK_CRAWL_GAITS"
 		options.walk_speed=2900
 		return {"RCP_UPPER_BODY","RCP_LOWER_BODY","RCP_NECK","RCP_HEAD"}
@@ -336,6 +355,12 @@ body_base_fun=body_base_fun or {
 		options.walk_speed=900
 		return {"RCP_BASIC_BODY_STANCE","RCP_NECK","RCP_HEAD","RCP_TAIL"}
 	end,
+	BIPEDAL_DINOSAUR=function(rcp,options)
+		options.pcg_layering_base="BEAST_BIPEDAL_DINOSAUR"
+		options.walk_var="STANDARD_BIPED_GAITS"
+		options.walk_speed=900
+		return {"RCP_UPPER_BODY","RCP_LOWER_BODY","RCP_NECK","RCP_HEAD","RCP_TWO_PART_ARMS","RCP_TWO_PART_LEGS"}
+	end,
 	HUMANOID=function(rcp,options)
 		options.pcg_layering_base="BEAST_HUMANOID"
 		options.walk_var="STANDARD_BIPED_GAITS"
@@ -343,7 +368,15 @@ body_base_fun=body_base_fun or {
 		return {"RCP_UPPER_BODY","RCP_LOWER_BODY","RCP_NECK","RCP_HEAD","RCP_TWO_PART_ARMS","RCP_TWO_PART_LEGS"}
 	end,
 	TWO_LEGS_NO_ARMS=function(rcp,options)
+		options.pcg_layering_base="BEAST_BIPEDAL_DINOSAUR"
+		options.use_armless_base=true
+		options.walk_var="STANDARD_WALKING_GAITS"
+		options.walk_speed=900
+		return {"RCP_UPPER_BODY","RCP_LOWER_BODY","RCP_NECK","RCP_HEAD","RCP_TWO_PART_LEGS"}
+	end,
+	TWO_LEGS_NO_ARMS_HUMANOID=function(rcp,options)
 		options.pcg_layering_base="BEAST_HUMANOID"
+		options.use_armless_base=true
 		options.walk_var="STANDARD_WALKING_GAITS"
 		options.walk_speed=900
 		return {"RCP_UPPER_BODY","RCP_LOWER_BODY","RCP_NECK","RCP_HEAD","RCP_TWO_PART_LEGS"}
@@ -352,18 +385,18 @@ body_base_fun=body_base_fun or {
 
 pcg_layering_base_info={
 	BEAST_AMORPHOUS={shell_pos="BACK"},
-	BEAST_SNAKE={shell_pos="BACK"},
-	BEAST_WORM_LONG={shell_pos="BACK"},
-	BEAST_WORM_SHORT={shell_pos="BACK"},
-	BEAST_INSECT={shell_pos="BACK"},
-	BEAST_SPIDER={shell_pos="FRONT"},
-	BEAST_SCORPION={tail=true,shell_pos="FRONT"},
-	BEAST_BIPEDAL_DINOSAUR={tail=true,shell_pos="BACK"},
-	BEAST_HUMANOID={tail=true,shell_pos="BACK"},
-	BEAST_FRONT_GRASP={tail=true,shell_pos="BACK"},
-	BEAST_QUADRUPED_BULKY={tail=true,shell_pos="FRONT"},
-	BEAST_QUADRUPED_SLINKY={tail=true,shell_pos="FRONT"},
-	BEAST_WALRUS={tail=true,shell_pos="BACK"},
+	BEAST_SNAKE={shell_pos="BACK",lacy_wings={"BACK","FRONT"},feathered_wings={"BACK","FRONT"},bat_wings={"BACK","FRONT"}},
+	BEAST_WORM_LONG={shell_pos="BACK",lacy_wings={"BACK","FRONT"},feathered_wings={"BACK","FRONT"},bat_wings={"BACK","FRONT"}},
+	BEAST_WORM_SHORT={shell_pos="BACK",lacy_wings={"BACK","FRONT"},feathered_wings={"BACK","FRONT"},bat_wings={"BACK","FRONT"}},
+	BEAST_INSECT={shell_pos="FRONT",lacy_wings={"BACK","FRONT"},feathered_wings={"BACK","FRONT"},bat_wings={"BACK","FRONT"}},
+	BEAST_SPIDER={shell_pos="FRONT",lacy_wings={"BACK","FRONT"},feathered_wings={"BACK","FRONT"},bat_wings={"BACK","FRONT"}},
+	BEAST_SCORPION={tail=true,shell_pos="FRONT",lacy_wings={"BACK","FRONT"},feathered_wings={"BACK","FRONT"},bat_wings={"BACK","FRONT"}},
+	BEAST_BIPEDAL_DINOSAUR={tail=true,shell_pos="FRONT",lacy_wings={"BACK","FRONT"},feathered_wings={"BACK","FRONT"},bat_wings={"BACK","FRONT"}},
+	BEAST_HUMANOID={tail=true,shell_pos="BACK",lacy_wings={"BACK","FRONT"},feathered_wings={"BACK","FRONT"},bat_wings={"BACK","FRONT"}},
+	BEAST_FRONT_GRASP={tail=true,shell_pos="BACK",lacy_wings={"BACK","FRONT"},feathered_wings={"BACK","FRONT"},bat_wings={"BACK","FRONT"}},
+	BEAST_QUADRUPED_BULKY={tail=true,shell_pos="FRONT",lacy_wings={"BACK","FRONT"},feathered_wings={"BACK","FRONT"},bat_wings={"BACK","FRONT"}},
+	BEAST_QUADRUPED_SLINKY={tail=true,shell_pos="FRONT",lacy_wings={"BACK","FRONT"},feathered_wings={"BACK","FRONT"},bat_wings={"BACK","FRONT"}},
+	BEAST_WALRUS={tail=true,shell_pos="BACK",lacy_wings={"BACK","FRONT"},feathered_wings={"BACK","FRONT"},bat_wings={"BACK","FRONT"}},
 }
 
 random_creature_types=random_creature_types or {
@@ -861,20 +894,20 @@ random_creature_types=random_creature_types or {
 	CRUSTACEAN_SHRIMP={
 		name_string="shrimp",
 		tile='S',
-		body_base="TEN_LEGGED",
+		body_base="TEN_LEGGED_LOBSTERFORM",
 		c_class="CHITIN_EXO",
 		must_have_antennae=true,
-		must_have_tail=true,
+		must_have_crustacean_tail=true,
 		min_size=70000,
 		weight=250
 		},
 	CRUSTACEAN_LOBSTER={
 		name_string="lobster",
 		tile=157,
-		body_base="EIGHT_LEGGED",
+		body_base="EIGHT_LEGGED_LOBSTERFORM",
 		c_class="CHITIN_EXO",
 		must_have_grabbing_claws=true,
-		must_have_tail=true,
+		must_have_crustacean_tail=true,
 		min_size=70000,
 		weight=250
 		},
@@ -1229,6 +1262,9 @@ random_creature_types=random_creature_types or {
 		body_base="QUADRUPED",
 		c_class="REPTILE",
 		must_have_tail=true,
+		must_have_quills=true,
+		must_have_plates=true,
+		cannot_have_shell=true,
 		min_size=500000,
 		front_digits=4,
 		rear_digits=4,
@@ -1248,7 +1284,7 @@ random_creature_types=random_creature_types or {
 	DINOSAUR_THEROPOD={--saurischia:X
 		name_string="theropod",
 		tile='T',
-		body_base="HUMANOID",
+		body_base="BIPEDAL_DINOSAUR",
 		c_class="REPTILE_FEATHERED",--theropod feathers
 		must_have_tail=true,
 		min_size=500000,
@@ -1260,7 +1296,7 @@ random_creature_types=random_creature_types or {
 	DINOSAUR_IGUANODONT={--ornithischia:cerapoda:ornithopoda
 		name_string="iguanodont",
 		tile='I',
-		body_base="HUMANOID",
+		body_base="BIPEDAL_DINOSAUR",
 		c_class="REPTILE",
 		must_have_tail=true,
 		min_size=500000,
@@ -1287,6 +1323,8 @@ random_creature_types=random_creature_types or {
 		body_base="QUADRUPED",
 		c_class="REPTILE",
 		must_have_tail=true,
+		must_have_plates=true,
+		cannot_have_shell=true,
 		min_size=500000,
 		front_digits=4,
 		rear_digits=4,
@@ -2465,7 +2503,7 @@ random_creature_types=random_creature_types or {
 	MAMMAL_FRUIT_BAT={
 		name_string="fruit bat",
 		tile='F',
-		body_base="TWO_LEGS_NO_ARMS",
+		body_base="TWO_LEGS_NO_ARMS_HUMANOID",
 		c_class="MAMMAL",
 		must_have_skin_wings=true,
 		min_size=70000,
@@ -2476,7 +2514,7 @@ random_creature_types=random_creature_types or {
 	MAMMAL_BAT={
 		name_string="bat",
 		tile='B',
-		body_base="TWO_LEGS_NO_ARMS",
+		body_base="TWO_LEGS_NO_ARMS_HUMANOID",
 		c_class="MAMMAL",
 		must_have_skin_wings=true,
 		min_size=70000,
