@@ -105,7 +105,7 @@ class init_displayst
 enum InitMediaFlag
 {
 	INIT_MEDIA_FLAG_SOUND_OFF,
-	INIT_MEDIA_FLAG_UNUSED_01_02,
+	INIT_MEDIA_FLAG_PORTABLE_MODE,
 	INIT_MEDIA_FLAG_COMPRESS_SAVES,
 	INIT_MEDIA_FLAGNUM
 };
